-- ===========================================================================
-- AClock is a clock that shows coordinated universal time 
-- (e.g. 2016/251/10:05/39Z)
-- Copyright (C) 2016-2016 Eric T. Hughes (a.k.a. "Eboneezur")
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
-- ===========================================================================
-- ===========================================================================
--- Class TPower
--
-- This class does little.  Not sure why I thought I needed to simulate a 
-- power button for a clock.
-- ===========================================================================
do
  local nIsOnce = 0
  local nIsOn   = 1

-- -------------------------------------------------------------------------
--- Method ON
--
-- This method turns on the clock
-- -------------------------------------------------------------------------
  local function ON()
    ClockFrameText:SetText("AClock ON")
    nIsOn = 1
    DEFAULT_CHAT_FRAME:AddMessage("[AClock] says: Power is turned on.", 0.0, 0.85, 0.0)

    if (nil ~= Broker_AClock.obj) then
      Broker_AClock.obj.icon = "Interface\\Addons\\AClock\\images\\bit_yes"
      Broker_AClock.obj.label = "AClock"
      Broker_AClock.obj.text = "ON"
    end
  end

-- -------------------------------------------------------------------------
--- Method OFF
--
-- This method turns off the clock
-- -------------------------------------------------------------------------
  local function OFF()
    ClockFrameText:SetText("AClock OFF")
    nIsOn = 0
    DEFAULT_CHAT_FRAME:AddMessage("[AClock] says: Power is turned off.", 0.0, 0.85, 0.0)

    if (nil ~= Broker_AClock.obj) then
      Broker_AClock.obj.icon = "Interface\\Addons\\AClock\\images\\bit_default"
      Broker_AClock.obj.label = "AClock"
      Broker_AClock.obj.text = "OFF"
    end
  end
  
-- -------------------------------------------------------------------------
--- Method HIDE
--
-- This method hides the clock
-- -------------------------------------------------------------------------
  local function HIDE()
    if (nil ~= Broker_AClock.obj) then
      Broker_AClock.obj.icon = "Interface\\Addons\\AClock\\images\\bit_default"
      Broker_AClock.obj.label = "AClock"
      Broker_AClock.obj.text = "Hide"
    end
  end

  
-- -------------------------------------------------------------------------
--- Method IsOn
--
-- This method returns 1 when the power is on or 0 when the power is off.
-- -------------------------------------------------------------------------
  local function IsOn()
    return nIsOn
  end

-- -------------------------------------------------------------------------
--- Method Toggle
--
-- This method determines whether to turn on or off the power.
-- @param self The given TPower object.
-- -------------------------------------------------------------------------
  local function Toggle(self)
    self.m_vecLocation.nIsInInstance, 
    self.m_vecLocation.strInstanceType = IsInInstance()

    if   (1      == self.m_vecLocation.nIsInInstance)   then
      if ("raid" == self.m_vecLocation.strInstanceType) then
        self.ON()
      else
        if (1 == nIsOn) then
          self.OFF()
        end
      end
    else
      if (1 == nIsOn) then
        ClockFrameText:SetText("AClock OFF")

        if   (1 == nIsOnce) then
        else
          DEFAULT_CHAT_FRAME:AddMessage(string.format("[AClock] says: Type /aclock show, /aclock hide, or /aclock config."), 0.0, 0.85, 0.0)
          nIsOnce = 1
        end

        nIsOn = 0
        DEFAULT_CHAT_FRAME:AddMessage("[AClock] says: The clock is hidden.", 0.0, 0.85, 0.0)
      end
    end

  end

-- -------------------------------------------------------------------------
--- Method Update
--
-- This method Updates the zone.
-- @param self The given TPower object.
-- -------------------------------------------------------------------------
  local function Update(self)
    self.m_vecLocation.strZone    = GetZoneText()
    self.m_vecLocation.strSubZone = GetSubZoneText()
    SetMapToCurrentZone()
  end


-- -------------------------------------------------------------------------
--- Method GetType
--
-- This method returns the type of this class.
-- -------------------------------------------------------------------------
  local function GetType()
      return "TPower"
   end

-- -------------------------------------------------------------------------
-- Method new_TPower
--
-- This method creates an object of this class.
-- -------------------------------------------------------------------------
  function new_TPower()

      local obj =
      {
        GetType = GetType,
        IsOn    = IsOn,
        OFF     = OFF,
        ON      = ON,
        HIDE    = HIDE,
        Toggle  = Toggle,
        Update  = Update,

        m_vecLocation =
        {
          strInstanceType = "",
          strZone         = "",
          strSubZone      = "",
          nX              = 0.0,
          nY              = 0.0,
          nIsInInstance   = 1,
        },
      }

      return obj
  end 
end


-- ===========================================================================
--- Class TSemaphore
--
-- This class defines a counting semaphore.
-- ===========================================================================
do
-- -------------------------------------------------------------------------
--- Method IsLocked
--
-- This method returns true if this counting semaphore has a lock.
-- @param self The given TSemaphore object.
-- -------------------------------------------------------------------------
  local function IsLocked(self)
    if 1 > self.nLockCount then
      return false
    else
      return true
    end
  end

-- -------------------------------------------------------------------------
--- Method IsUnLocked
--
-- This method returns true if this counting semaphore does not have a lock.
-- @param self The given TSemaphore object.
-- -------------------------------------------------------------------------
  local function IsUnLocked(self)
    if 0 < self.nLockCount then
      return false
    else
      return true
    end
  end

-- -------------------------------------------------------------------------
--- Method Lock
--
-- This method locks this counting semaphore and increases the lock count.
-- @param self The given TSemaphore object.
-- -------------------------------------------------------------------------
  local function Lock(self)
    self.nLockCount = self.nLockCount + 1
  end

-- -------------------------------------------------------------------------
--- Method UnLock
--
-- This method unlocks this counting semaphore and decreases the lock count.
-- @param self The given TSemaphore object.
-- -------------------------------------------------------------------------
  local function UnLock(self)
    if (0 < self.nLockCount) then
      self.nLockCount = self.nLockCount - 1
    end
  end

-- -------------------------------------------------------------------------
--- Method GetType
--
-- This method returns the type of this class.
-- -------------------------------------------------------------------------
  local function GetType()
    return "TSemaphore"
  end

-- -------------------------------------------------------------------------
--- Method new_TSemaphore
--
-- This method creates an object of this class.
-- -------------------------------------------------------------------------
  function new_TSemaphore()
    local obj =
    {
      GetType    = GetType,
      IsLocked   = IsLocked,
      IsUnLocked = IsUnLocked,
      Lock       = Lock,
      UnLock     = UnLock,
      nLockCount = 0,
    }

    return obj
  end 
end



-- ===========================================================================
--- Class TClock
--
-- This class encapsulates the WoW API.
-- ===========================================================================
do
  -- Protect event handlers against WoW patches and programmer error.

  local semEventPLAYER_ENTERING_WORLD = new_TSemaphore()
  local nIsDisplayed = 1  -- Default to displayed

-- -------------------------------------------------------------------------
--- Method Hide
--
-- This method hides the clock.
-- @param self The given TCalc object.
-- -------------------------------------------------------------------------
   local function Hide(self)
      ClockFrame:Hide()
      nIsDisplayed = 0
   end

-- -------------------------------------------------------------------------
--- Method IsDisplayed
--
-- This method returns 1 when the status button is shown or 
-- 0 when it is hidden.
-- @param self The given TCalc object.
-- -------------------------------------------------------------------------
   local function IsDisplayed(self)
     return nIsDisplayed
   end

-- -------------------------------------------------------------------------
--- Method Show
--
-- This method shows the clock.
-- @param self The given TClock object.
-- -------------------------------------------------------------------------
   local function Show(self)
      ClockFrame:Show()
      nIsDisplayed = 1
   end

-- -------------------------------------------------------------------------
--- Method Toggle
--
-- This method shows/hides the clock.
-- @param The given TClock object.
-- -------------------------------------------------------------------------
  local function Toggle(self)
    if (0 == nIsDisplayed) then Show(self) else Hide(self) end
  end

-- -------------------------------------------------------------------------
--- Method OnLoad
--
-- This method registers WoW API events.
-- @param frame The given status button window.
-- -------------------------------------------------------------------------
  local function OnLoad(frame)
    frame:RegisterEvent("PLAYER_ENTERING_WORLD")
    frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    frame:RegisterForDrag("LeftButton")

    if (0 == nIsDisplayed) then Hide(self) else Show(self) end
  end


-- -------------------------------------------------------------------------
--- Method HandleEventPLAYER_ENTERING_WORLD
--
-- This method handles WoW API event "PLAYER_ENTERING_WORLD".
-- -------------------------------------------------------------------------
  local function HandleEventPLAYER_ENTERING_WORLD()
    if semEventPLAYER_ENTERING_WORLD:IsUnLocked() then
      semEventPLAYER_ENTERING_WORLD:Lock()

      tblPower1:Toggle()

      semEventPLAYER_ENTERING_WORLD:UnLock()
    end
  end




-- -------------------------------------------------------------------------
--- Method OnEvent
--
-- This method handles WoW API events.
-- @param frame The given status button window.
-- @param event The given WoW event.
-- -------------------------------------------------------------------------
  local function OnEvent(frame, event, ...)
        if event == "PLAYER_ENTERING_WORLD" then HandleEventPLAYER_ENTERING_WORLD()
    end
  end    

  local function SetFormat(self, a_strExpression)
    self.strFormat = a_strExpression
  end
  
  local numTimerDelay = 0.5
  local numCounter    = 0
  local tblDateUTC
  local tblDateLocal
  local strDate

-- -------------------------------------------------------------------------
-- Method OnUpdate
--
-- This method serves as a timer event.
--
-- Note: numHour, numMinute = GetGameTime() returns the time of the server;
--       however, the time can warp when the server is switched to an
--       instance server (e.g. arena, battleground, or dungeon).
-- -------------------------------------------------------------------------
  local function OnUpdate(self)
    numCounter = numCounter + 0.01
 
    if numCounter >= numTimerDelay then 
      if ("local" == self.strFormat) then	
        tblDateLocal = date("*t")	  
        strDate = string.format(" %04s/%03s/%02s:%02s:%02s   ", tblDateLocal.year, tblDateLocal.yday, tblDateLocal.hour, tblDateLocal.min, tblDateLocal.sec)
      else
        tblDateUTC = date("!*t")	  
        strDate = string.format(" %04s/%03s/%02s:%02s:%02sZ ", tblDateUTC.year, tblDateUTC.yday, tblDateUTC.hour, tblDateUTC.min, tblDateUTC.sec)	  
      end
	  
      ClockFrameText:SetText(strDate)
      numCounter = 0
    end
  end
  
  local strNowPlayingSoundFile = nil
  
  local function OnUpdatePlayAmbientSounds()      
      if (15 == tblDateUTC.min) then
        if (0 == tblDateUTC.sec) then
          if (nil == strNowPlayingSoundFile) then
            strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\onequarterchime.ogg"
            PlaySoundFile(strNowPlayingSoundFile, "Ambience")
            -- print(strNowPlayingSoundFile)
          end
        end
        
        if (4 == tblDateUTC.sec) then
          strNowPlayingSoundFile = nil
        end
      elseif (30 == tblDateUTC.min) then
        if (0 == tblDateUTC.sec) then
          if (nil == strNowPlayingSoundFile) then
            strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\twoquarterchime.ogg"
            PlaySoundFile(strNowPlayingSoundFile, "Ambience")
            -- print(strNowPlayingSoundFile)
          end
        end
        
        if (8 == tblDateUTC.sec) then
          strNowPlayingSoundFile = nil
        end
      elseif (45 == tblDateUTC.min) then
        if (0 == tblDateUTC.sec) then
          if (nil == strNowPlayingSoundFile) then
            strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\threequarterchime.ogg"
            PlaySoundFile(strNowPlayingSoundFile, "Ambience")
            -- print(strNowPlayingSoundFile)
          end
        end
          
        if (11 == tblDateUTC.sec) then
          strNowPlayingSoundFile = nil
        end
      elseif (0 == tblDateUTC.min) then
        if (0 == tblDateUTC.sec) then
          if (nil == strNowPlayingSoundFile) then
            strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\fourquarterchime.ogg"
            PlaySoundFile(strNowPlayingSoundFile, "Ambience")
            -- print(strNowPlayingSoundFile)
          end
        end
        if (14 == tblDateUTC.sec) then
          strNowPlayingSoundFile = nil
        end
        
        if (15 == tblDateUTC.sec) then
          if (nil == strNowPlayingSoundFile) then
                if ( 1 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\onestrike.ogg"
            elseif ( 2 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\twostrikes.ogg"
            elseif ( 3 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\threestrikes.ogg"
            elseif ( 4 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\fourstrikes.ogg"
            elseif ( 5 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\fivestrikes.ogg"
            elseif ( 6 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\sixstrikes.ogg"
            elseif ( 7 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\sevenstrikes.ogg"
            elseif ( 8 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\eightstrikes.ogg"
            elseif ( 9 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\ninestrikes.ogg"
            elseif (10 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\tenstrikes.ogg"
            elseif (11 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\elevenstrikes.ogg"
            elseif (12 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\twelvestrikes.ogg"
            elseif (13 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\onestrike.ogg"
            elseif (14 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\twostrikes.ogg"
            elseif (15 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\threestrikes.ogg"
            elseif (16 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\fourstrikes.ogg"
            elseif (17 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\fivestrikes.ogg"
            elseif (18 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\sixstrikes.ogg"
            elseif (19 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\sevenstrikes.ogg"
            elseif (20 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\eightstrikes.ogg"
            elseif (21 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\ninestrikes.ogg"
            elseif (22 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\tenstrikes.ogg"
            elseif (23 == tblDateUTC.hour) then strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\elevenstrikes.ogg"
            else
              strNowPlayingSoundFile = "Interface\\Addons\\AClock\\sounds\\twelvestrikes.ogg"
            end
            
            PlaySoundFile(strNowPlayingSoundFile, "Ambience")
            -- print(strNowPlayingSoundFile)
          end
        end
      elseif (1 == tblDateUTC.min) then
        if (6 == tblDateUTC.sec) then
          strNowPlayingSoundFile = nil
        end
      end      
  end
    
-- -------------------------------------------------------------------------
--- Method GetType
--
-- This method returns the type of this class.
-- -------------------------------------------------------------------------
  local function GetType()
    return "TClock"
  end

-- -------------------------------------------------------------------------
--- Method new_TClock
--
-- This method creates an object of this class.
-- -------------------------------------------------------------------------
  function new_TClock()
    local obj =
    {
        Hide                      = Hide,
        IsDisplayed               = IsDisplayed,
        OnEvent                   = OnEvent,
        OnLoad                    = OnLoad,
        OnUpdate                  = OnUpdate,
        OnUpdatePlayAmbientSounds = OnUpdatePlayAmbientSounds,        
        SetFormat                 = SetFormat,
        Show                      = Show,
        Toggle                    = Toggle,
        strFormat                 = "!*t",
    }

    obj.panel = CreateFrame( "Frame", "MyAddonPanel", UIParent )
    obj.panel.name = "AClock"

    local strCreationDate
    
    tblDateUTC = date("!*t")
    strCreationDate = string.format(" %04s/%03s/%02s:%02s:%02sZ ", tblDateUTC.year, tblDateUTC.yday, tblDateUTC.hour, tblDateUTC.min, tblDateUTC.sec)	  
    
    local strTitle = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge")
    local strOverview = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine2 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine3 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine4 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine5 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine6 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine098 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine099 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine100 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine101 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine102 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine103 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine104 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine105 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine106 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine107 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine108 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strTableHeaderCol1 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    local strTableHeaderCol2 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    local strCell11 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell12 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell21 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell22 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell31 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell32 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell41 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell42 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell51 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell52 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell61 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell62 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")

    strTitle:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -15)
    strOverview:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -45)
    strLine2:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -60)
    strLine3:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -75)
    strLine4:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -90)
    strLine5:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -105)
    strLine6:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -120)
    strTableHeaderCol1:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -150)
    strTableHeaderCol2:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -150)
    strCell11:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -165)
    strCell21:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -180)
    strCell31:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -195)
    strCell41:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -210)
    strCell51:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -225)
    strCell61:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -240)
    strCell12:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -165)
    strCell22:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -180)
    strCell32:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -195)
    strCell42:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -210)
    strCell52:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -225)
    strCell62:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -240)
    strLine098:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -260)
    strLine099:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -275)
    strLine100:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -290)
    strLine101:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -305)
    strLine102:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -320)
    strLine103:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -335)
    strLine104:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -350)
    strLine105:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -365)
    strLine106:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -380)
    strLine107:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -395)
    strLine108:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -410)

    strTitle:SetTextColor(0.0, 0.85, 0.0)
    strOverview:SetTextColor(1.0, 1.0, 1.0)
    strLine2:SetTextColor(1.0, 1.0, 1.0)
    strLine3:SetTextColor(1.0, 1.0, 1.0)
    strLine4:SetTextColor(1.0, 1.0, 1.0)
    strLine5:SetTextColor(1.0, 1.0, 1.0)
    strLine6:SetTextColor(1.0, 1.0, 1.0)
    strCell11:SetTextColor(1.0, 1.0, 1.0)
    strCell21:SetTextColor(1.0, 1.0, 1.0)
    strCell31:SetTextColor(1.0, 1.0, 1.0)
    strCell41:SetTextColor(1.0, 1.0, 1.0)
    strCell51:SetTextColor(1.0, 1.0, 1.0)
    strCell61:SetTextColor(1.0, 1.0, 1.0)
    strCell12:SetTextColor(1.0, 1.0, 1.0)
    strCell22:SetTextColor(1.0, 1.0, 1.0)
    strCell32:SetTextColor(1.0, 1.0, 1.0)
    strCell42:SetTextColor(1.0, 1.0, 1.0)
    strCell52:SetTextColor(1.0, 1.0, 1.0)
    strCell62:SetTextColor(1.0, 1.0, 1.0)
    strLine098:SetTextColor(1.0, 1.0, 1.0)
    strLine099:SetTextColor(1.0, 1.0, 1.0)
    strLine100:SetTextColor(1.0, 1.0, 1.0)
    strLine101:SetTextColor(1.0, 1.0, 1.0)
    strLine102:SetTextColor(1.0, 1.0, 1.0)
    strLine103:SetTextColor(1.0, 1.0, 1.0)
    strLine104:SetTextColor(1.0, 1.0, 1.0)
    strLine105:SetTextColor(1.0, 1.0, 1.0)
    strLine106:SetTextColor(1.0, 1.0, 1.0)
    strLine107:SetTextColor(1.0, 1.0, 1.0)
    strLine108:SetTextColor(1.0, 1.0, 1.0)
    strTitle:SetText("AClock V" ..GetAddOnMetadata("AClock", "Version" ) .. "   by " .. GetAddOnMetadata("AClock", "Author"))
    strOverview:SetText("Do you need to coordinate a raid group across multiple time zones?")
    strLine2:SetText(string.format("AClock displays coordinated universal time (e.g. %s)", strCreationDate))

    strTableHeaderCol1:SetText("Slash Command")
    strTableHeaderCol2:SetText("Description")
    strCell31:SetText("/aclock show")
    strCell32:SetText("shows the clock")             
    strCell41:SetText("/aclock hide")
    strCell42:SetText("hides the clock")             
    strCell51:SetText("/aclock config")
    strCell52:SetText("display configuration options")             


    strLine098:SetText(string.format("Copyright (C) 2016-%04s Eric T. Hughes", tblDateUTC.year))
    strLine099:SetText("")
    strLine100:SetText("This add-on is free software: you can redistribute it and/or modify")
    strLine101:SetText("it under the terms of the GNU General Public License as published by")
    strLine102:SetText("the Free Software Foundation, either version 3 of the License, or")
    strLine103:SetText("(at your option) any later version.")
    strLine104:SetText("")
    strLine105:SetText("This add-on is distributed in the hope that it will be useful,")
    strLine106:SetText("but WITHOUT ANY WARRANTY; without even the implied warranty of")
    strLine107:SetText("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU")
    strLine108:SetText("General Public License for more details. <http://www.gnu.org/licenses/>")

    obj.panel.strTitle = strTitle
    obj.panel.strOverview = strOverview
    obj.panel.strLine2 = strLine2
    obj.panel.strTableHeader = strTableHeader
    obj.panel.strCell11 = strCell11
    obj.panel.strCell21 = strCell21
    obj.panel.strCell31 = strCell31
    obj.panel.strCell41 = strCell41
    obj.panel.strCell51 = strCell51
    obj.panel.strCell61 = strCell61
    obj.panel.strCell12 = strCell12
    obj.panel.strCell22 = strCell22
    obj.panel.strCell32 = strCell32
    obj.panel.strCell42 = strCell42    
    obj.panel.strCell52 = strCell52    
    obj.panel.strCell62 = strCell62    
    obj.panel.strLine098 = strLine098
    obj.panel.strLine099 = strLine099
    obj.panel.strLine100 = strLine100
    obj.panel.strLine101 = strLine101
    obj.panel.strLine102 = strLine102
    obj.panel.strLine103 = strLine103
    obj.panel.strLine104 = strLine104
    obj.panel.strLine105 = strLine105
    obj.panel.strLine106 = strLine106
    obj.panel.strLine107 = strLine107
    obj.panel.strLine108 = strLine108
    obj.panel:Hide()
    InterfaceOptions_AddCategory(obj.panel);
    return obj
  end 
end


tblPower1 = new_TPower()
tblClock  = new_TClock()


-- -------------------------------------------------------------------------
--- Function SlashCmdClockHandler
--
-- This function evaluates a given expression.
-- @param a_strExpression The given expression.
-- -------------------------------------------------------------------------
function SlashCmdClockHandler(a_strExpression)
  if (nil ~= Broker_AClock) then
    Broker_AClock.obj.label = "AClock"
  end

  if a_strExpression == "" or a_strExpression == "?" or a_strExpression == "help" or a_strExpression == "HELP" or a_strExpression == "Help" then
    strMessage = string.format("[AClock] says: Here are my slash commands:\n%3s\n%3s\n%3s", 
          "/aclock show  - shows the clock",
          "/aclock hide   - hides the clock", 
          "/aclock config - display configuration options")
    DEFAULT_CHAT_FRAME:AddMessage(strMessage, 0.0, 0.85, 0.0)
		
    if (nil ~= Broker_AClock) then
      Broker_AClock.obj.text = "?"
    end
  else
    if a_strExpression == "config" or a_strExpression == "CONFIG" or a_strExpression == "Config" then
      if (nil ~= Broker_AClock) then
        Broker_AClock.obj.text = "CONFIG"
      end

      InterfaceOptionsFrame_OpenToCategory(tblClock.panel)
      InterfaceOptionsFrame_OpenToCategory(tblClock.panel.name)

    else
      if a_strExpression == "show" or a_strExpression == "SHOW" or a_strExpression == "Show" then
        tblClock:Show()
        if (nil ~= Broker_AClock) then
          Broker_AClock.obj.text = "SHOW"
        end
      else 
        if a_strExpression == "hide" or a_strExpression == "HIDE" or a_strExpression == "Hide" then
          tblClock:Hide()      
          if (nil ~= Broker_AClock) then
            Broker_AClock.obj.text = "HIDE"
          end
        else
          tblClock:SetFormat(a_strExpression)
        end
      end
    end
  end	
end




SLASH_ACLOCK1, SLASH_ACLOCK2, SLASH_ACLOCK3, SLASH_ACLOCK4  = "/aclock", "/aclock show", "/aclock hide", "/aclock config"
SlashCmdList["ACLOCK"] = SlashCmdClockHandler
